@@ -327,14 +327,14 @@ impl BackendLockGuard<'_> {
         let mut import_result = Vec::with_capacity(secret_keys);
         let mut keys = Vec::with_capacity(secret_keys);
         for ka in cert.keys().secret() {
-            log::debug!("Adding {} (cert: {})", ka.keyid(), cert.keyid());
+            log::debug!("Adding {} (cert: {})", ka.key().keyid(), cert.keyid());
 
-            let id = ka.fingerprint();
+            let id = ka.key().fingerprint();
 
-            match self.0.keys.entry(ka.fingerprint()) {
+            match self.0.keys.entry(ka.key().fingerprint()) {
                 Entry::Occupied(oe) => {
                     log::debug!("{} ({}) already present",
-                                ka.keyid(), cert.keyid());
+                                ka.key().keyid(), cert.keyid());
 
                     let sk = oe.get();
 
@@ -347,7 +347,7 @@ impl BackendLockGuard<'_> {
                         = variants.iter().position(|k| &k.key == ka.key())
                     {
                         log::debug!("{} ({}): variant unchanged",
-                                    ka.keyid(), cert.keyid());
+                                    ka.key().keyid(), cert.keyid());
 
                         // Move it to the end.  When exporting, we
                         // want to export the most recently imported
@@ -357,7 +357,7 @@ impl BackendLockGuard<'_> {
                         ImportStatus::Unchanged
                     } else {
                         log::debug!("{} ({}): new variant",
-                                    ka.keyid(), cert.keyid());
+                                    ka.key().keyid(), cert.keyid());
 
                         variants.push(KeyVariant {
                             key: ka.key().clone(),
@@ -504,7 +504,7 @@ impl backend::Backend for Backend {
 
         let backend = self.inner.lock().await;
 
-        backend.certs.get(&id.parse()?)
+        backend.certs.get(&id.parse::<Fingerprint>()?)
             .map(|sc| {
                 Box::new(sc.clone())
                     as Box<dyn backend::DeviceHandle + Send + Sync + 'a>
@@ -519,7 +519,7 @@ impl backend::Backend for Backend {
 
         let backend = self.lock().await;
 
-        backend.keys.get(&id.parse()?)
+        backend.keys.get(&id.parse::<Fingerprint>()?)
             .map(|sk| {
                 Box::new(sk.clone()) as Box<dyn backend::KeyHandle + Send + Sync + 'a>
             })
@@ -757,7 +757,7 @@ impl backend::KeyHandle for Key {
         for variant in kh.variants.iter() {
             let mut unlocked = variant.key.clone();
             match unlocked.secret_mut()
-                .decrypt_in_place(variant.key.pk_algo(), password)
+                .decrypt_in_place(&variant.key, password)
             {
                 Ok(()) => {
                     log::trace!("KeyHandle::unlock({}): Unlocked", self.fpr);
@@ -997,7 +997,7 @@ impl backend::KeyHandle for Key {
                 format!("Reading {}", variant.path.display())
             })?;
         let orig_key = cert.keys().find(|k| {
-            k.fingerprint() == new_key.fingerprint()
+            k.key().fingerprint() == new_key.fingerprint()
         });
 
         let orig_key = if let Some(orig_key) = orig_key {
@@ -1015,12 +1015,12 @@ impl backend::KeyHandle for Key {
             cert.insert_packets(new_key.clone().role_into_primary())
                 .with_context(|| {
                     format!("Updating {}", variant.path.display())
-                })?
+                })?.0
         } else {
             cert.insert_packets(new_key.clone().role_into_subordinate())
                 .with_context(|| {
                     format!("Updating {}", variant.path.display())
-                })?
+                })?.0
         };
 
         let mut tmp_filename = variant.path.clone();
@@ -1078,7 +1078,7 @@ impl backend::KeyHandle for Key {
                 })?;
 
             let orig_key = cert.keys().find(|k| {
-                k.fingerprint() == fpr
+                k.key().fingerprint() == fpr
             });
             let orig_key = if let Some(orig_key) = orig_key {
                 orig_key
@@ -1094,12 +1094,12 @@ impl backend::KeyHandle for Key {
                 cert.insert_packets(kh.public_key.clone().role_into_primary())
                     .with_context(|| {
                         format!("Updating {}", variant.path.display())
-                    })?
+                    })?.0
             } else {
                 cert.insert_packets(kh.public_key.clone().role_into_subordinate())
                     .with_context(|| {
                         format!("Updating {}", variant.path.display())
-                    })?
+                    })?.0
             };
 
             if ! cert.is_tsk() {
@@ -1180,6 +1180,7 @@ impl backend::KeyHandle for Key {
 
 #[cfg(test)]
 mod tests {
+    use openpgp::KeyHandle;
     use super::*;
 
     use backend::test_framework;
@@ -1231,19 +1232,18 @@ mod tests {
         import_cert(&mut backend, &cert).await;
 
         let msg: &[u8] = test_framework::data::message("bob.asc");
-        let keyid: sequoia_openpgp::KeyID
-            = "0A4BA2249168D779".parse().expect("valid");
+        let keyid = KeyHandle::KeyID("0A4BA2249168D779".parse().expect("valid"));
 
         test_framework::try_decrypt(
             &mut backend, msg, Some(b"hi bob\n"),
-            &keyid, Some(Password::from("bob"))).await.unwrap();
+            keyid.clone(), Some(Password::from("bob"))).await.unwrap();
         test_framework::try_decrypt(
             &mut backend, msg, Some(b"hi bob\n"),
-            &keyid, Some(Password::from("smithy"))).await.unwrap();
+            keyid.clone(), Some(Password::from("smithy"))).await.unwrap();
 
         assert!(test_framework::try_decrypt(
             &mut backend, msg, Some(b"hi bob\n"),
-            &keyid, Some(Password::from("sup3r s3cur3"))).await.is_err());
+            keyid, Some(Password::from("sup3r s3cur3"))).await.is_err());
 
         Ok(())
     }
@@ -1270,18 +1270,17 @@ mod tests {
         assert_eq!(cert.fingerprint(), cert2.fingerprint());
 
         let msg: &[u8] = test_framework::data::message("bob.asc");
-        let keyid: sequoia_openpgp::KeyID
-            = "0A4BA2249168D779".parse().expect("valid");
+        let keyid = KeyHandle::KeyID("0A4BA2249168D779".parse()?);
 
         eprintln!("Decrypting with bob (correct)");
         test_framework::try_decrypt(
             &mut backend, msg, Some(b"hi bob\n"),
-            &keyid, Some(Password::from("bob"))).await.unwrap();
+            keyid.clone(), Some(Password::from("bob"))).await.unwrap();
 
         eprintln!("Decrypting with sup3r s3cur3 (incorrect)");
         assert!(test_framework::try_decrypt(
             &mut backend, msg, Some(b"hi bob\n"),
-            &keyid, Some(Password::from("sup3r s3cur3"))).await.is_err());
+            keyid.clone(), Some(Password::from("sup3r s3cur3"))).await.is_err());
 
         Ok(())
     }
@@ -1351,17 +1350,17 @@ mod tests {
         let fpr_str = "86353DE523CB334DC0B1F0F00A4BA2249168D779";
         let fpr: sequoia_openpgp::Fingerprint
             = fpr_str.parse().expect("valid");
-        let keyid = sequoia_openpgp::KeyID::from(fpr);
+        let keyid = KeyHandle::KeyID(fpr.into());
 
         // Wrong password.
         assert!(test_framework::try_decrypt(
             &mut backend, msg, Some(b"hi bob\n"),
-            &keyid, Some(Password::from("sup3r s3cur3"))).await.is_err());
+            keyid.clone(), Some(Password::from("sup3r s3cur3"))).await.is_err());
 
         // Right password.
         test_framework::try_decrypt(
             &mut backend, msg, Some(b"hi bob\n"),
-            &keyid, Some(Password::from("bob"))).await.unwrap();
+            keyid.clone(), Some(Password::from("bob"))).await.unwrap();
 
         // Change the password.
         let mut key = backend.find_key(fpr_str)
@@ -1376,15 +1375,15 @@ mod tests {
         log::info!("Decrypting with wrong password:");
         assert!(test_framework::try_decrypt(
             &mut backend, msg, Some(b"hi bob\n"),
-            &keyid, Some(Password::from("sup3r s3cur3"))).await.is_err());
+            keyid.clone(), Some(Password::from("sup3r s3cur3"))).await.is_err());
         log::info!("Decrypting with old password:");
         assert!(test_framework::try_decrypt(
             &mut backend, msg, Some(b"hi bob\n"),
-            &keyid, Some(Password::from("smithy"))).await.is_err());
+            keyid.clone(), Some(Password::from("smithy"))).await.is_err());
         log::info!("Decrypting with new password:");
         test_framework::try_decrypt(
             &mut backend, msg, Some(b"hi bob\n"),
-            &keyid, Some(Password::from("new password"))).await.unwrap();
+            keyid.clone(), Some(Password::from("new password"))).await.unwrap();
 
         // Copy the backend's directory.
         let home = {
@@ -1405,17 +1404,17 @@ mod tests {
         // Wrong password.
         assert!(test_framework::try_decrypt(
             &mut new_backend, msg, Some(b"hi bob\n"),
-            &keyid, Some(Password::from("sup3r s3cur3"))).await.is_err());
+            keyid.clone(), Some(Password::from("sup3r s3cur3"))).await.is_err());
 
         // Old password.
         assert!(test_framework::try_decrypt(
             &mut new_backend, msg, Some(b"hi bob\n"),
-            &keyid, Some(Password::from("bob"))).await.is_err());
+            keyid.clone(), Some(Password::from("bob"))).await.is_err());
 
         // New password.
         test_framework::try_decrypt(
             &mut new_backend, msg, Some(b"hi bob\n"),
-            &keyid, Some(Password::from("new password"))).await.unwrap();
+            keyid.clone(), Some(Password::from("new password"))).await.unwrap();
 
         Ok(())
     }
